@@ -41,6 +41,7 @@
 #include "brpc/http_method.h"
 #include "brpc/event_dispatcher.h"
 #include "brpc/versioned_ref_with_id.h"
+#include "brpc/health_check_option.h"
 
 namespace brpc {
 namespace policy {
@@ -285,6 +286,7 @@ struct SocketOptions {
     int tcp_user_timeout_ms{ -1};
     // Tag of this socket
     bthread_tag_t bthread_tag{bthread_self_tag()};
+    HealthCheckOption hc_option;
 };
 
 // Abstractions on reading from and writing into file descriptors.
@@ -412,6 +414,10 @@ public:
     // Initialized by SocketOptions.health_check_interval_s.
     int health_check_interval() const { return _health_check_interval_s; }
 
+    const std::string& health_check_path() const { return _hc_option.health_check_path; }
+
+    int32_t health_check_timeout_ms() const {return _hc_option.health_check_timeout_ms; }
+
     // True if health checking is enabled.
     bool HCEnabled() const {
         // This fence makes sure that we see change of
@@ -980,6 +986,7 @@ private:
     int _tcp_user_timeout_ms;
 
     HttpMethod _http_request_method;
+    HealthCheckOption _hc_option;
 };
 
 } // namespace brpc
