@@ -28,6 +28,12 @@
 
 namespace bvar {
 
+static const size_t INVALID_SAMPLING_RANGE = 0;
+
+inline bool is_sampling_range_valid(size_t sampling_range) {
+    return sampling_range > 0;
+}
+
 // Containing the context for limiting sampling speed.
 struct CollectorSpeedLimit {
     // [Managed by Collector, don't change!]
@@ -115,7 +121,7 @@ inline size_t is_collectable(CollectorSpeedLimit* speed_limit) {
         const size_t sampling_range = speed_limit->sampling_range;
         // fast_rand is faster than fast_rand_in
         if ((butil::fast_rand() & (COLLECTOR_SAMPLING_BASE - 1)) >= sampling_range) {
-            return 0;
+            return INVALID_SAMPLING_RANGE;
         }
         return sampling_range;
     }
